/***************************************************************************
                          sgui_dialog.cpp  -  description
                             -------------------
    begin                : Son Feb 16 2003
    copyright            : (C) 2003 by Matthias Bach
    email                : marix@marix-world.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sgui_dialog.h"

#include <vector>
#include <iostream>

using namespace std;

#include <SDL/SDL.h>


//#define DEBUG


SGUI_Dialog::SGUI_Dialog(){
	x = 0;
	y = 0;
	
	backgroundColor.r = 0;
	backgroundColor.g = 0;
	backgroundColor.b = 0;
	backgroundColor.unused = 0;

	this->focusedObject = -1;

	this->visible = false;
}
SGUI_Dialog::~SGUI_Dialog(){
	// free objects
	for(unsigned int i; i < objects.size(); i++)
		if(objects[i] != NULL)
			delete objects[i];

	objects.clear();
}
/** Adds an object to the dialog */
int SGUI_Dialog::addObject(SGUI_Object * object){
	this->objects.push_back(object);

	object->setColorMask(this->colorMask.Rmask, this->colorMask.Gmask, this->colorMask.Bmask, this->colorMask.Amask);

	object->visible = this->visible;

	return 42;

}
/** Removes an Object from the dialog */
int SGUI_Dialog::removeObject(SGUI_Object * object){
	// find the object
	for(unsigned int i = 0; i < objects.size(); i++)
	{
		if(this->objects[i] == object)
		{
			// delete object

			object->focused = false;
			object->visible = false;

			this->objects.erase(this->objects.begin() + i);
			return 42;
		}
	}

	// object couln't be erased
	return -1;

}
/** Sets the size to the given values. Returns 42 on succes and -1 on failure */
int SGUI_Dialog::setSize(int w, int h){
		// if there is already something allocated, then error
	if(this->surface != NULL)
		return -1;

	this->pxHeight = h;
	this->pxWidth = w;

	return 42;
}

/** Renders the dialog and all of it's objects and returns the Surface. Therefore it creates a SDL_Surface if needed. The return value is NULL on failure */
SDL_Surface * SGUI_Dialog::draw(){
  // check whether a surface exists, if there is none, create one
  if(this->surface == NULL)
  {
  	this->surface = SDL_CreateRGBSurface(0, this->pxWidth, this->pxHeight, 32, this->colorMask.Rmask, this->colorMask.Gmask, this->colorMask.Bmask, this->colorMask.Amask);
  	// check wether surface could be created
  	if(this->surface == NULL)
  	{
			cerr << "Could not create SDL_Surface " << SDL_GetError() << endl;
			return NULL;
		}
	}

	SDL_Rect src, dest;

	// fill surface with background color
	dest.x = 0;
	dest.y = 0;
	dest.w = this->surface->w;
	dest.h = this->surface->h;

	Uint32 color = SDL_MapRGB(this->surface->format, this->backgroundColor.r, this->backgroundColor.g, this->backgroundColor.b);

	SDL_FillRect(this->surface, &dest, color);

	SDL_Surface * tmp;

	src.x = 0;
	src.y = 0;

	// draw the objects to the surface
	for(unsigned int i = 0; i < this->objects.size(); i++)
	{
		// only draw visible objects
  	if(this->objects[i]->visible)
  	{
  		tmp = this->objects[i]->draw();
  		dest.x = objects[i]->x;
  		dest.y = objects[i]->y;

  		src.w = tmp->w;
  		src.h = tmp->h;

  		dest.w = src.w;
  		dest.h = src.h;

    	SDL_BlitSurface(tmp, &src, this->surface, &dest);

  		// tmp does not need to be freed, as the objects will free it itself on destructi
  	}
	}
	// return surface
	return this->surface;
}
/** Handles the given event, mostly by passing it on to it's registered Objects */
int SGUI_Dialog::message(SDL_Event * event){
	int lastFocusedObject = this->focusedObject;

	// all of this does only make sense if there are objects
	if(this->objects.size() != 0)
	{
  	// if it was a mouse-click the focus may have changed
  	if(event->type == SDL_MOUSEBUTTONDOWN)
  	{
			// get click position
			int mouseX = event->button.x;
			int mouseY = event->button.y;


#ifdef DEBUG
			cout << "Mouse clicked (" << mouseX << "/" << mouseY << ")" << endl;
#endif

  		this->focusedObject = -1;
  		for(unsigned int i = 0; i < objects.size(); i++)
  		{
				// only check if object is visible
				if(this->objects[i]->visible)
  			{
    			// check whether event was for that object
       		// objects coords need to be corrected with dialog-coords
  				if(mouseX > this->objects[i]->x + this->x && mouseX < this->objects[i]->x + this->x + this->objects[i]->width()
  					&& mouseY > this->objects[i]->y + this->y && mouseY < this->objects[i]->y + this->y + this->objects[i]->height())
  				{
  					// this object is now focused
  					this->focusedObject = i;

  #ifdef DEBUG
  					cout << "Object " << i << " hit" << endl;
  #endif
  				}
  			}
  		}
  	}

  	// if it was a curser-key, and there is no object selected, or the selected object is not a text-field
   	// on DOWN-ARRAY, RIGHT-ARRAY and TAB focus the next object
    // on UP-ARRAY and LEFT-ARRAY go back to the last object
    // don't go to next object, if current focused is a textfield or a dialog
  	if(event->type == SDL_KEYDOWN)
  	{
  		if(event->key.keysym.sym == SDLK_DOWN || event->key.keysym.sym == SDLK_RIGHT || event->key.keysym.sym == SDLK_TAB)
  		{
  			if(this->focusedObject == -1)
  			{
  				this->focusedObject = 0;
  			}
  			else if(this->objects[this->focusedObject]->getType() != SGUI_TEXTFIELD && this->objects[this->focusedObject]->getType() != SGUI_DIALOG)
  			{
  				this->focusedObject++;
  				this->focusedObject %= this->objects.size();
  			}
    	}
  		else if(event->key.keysym.sym == SDLK_UP || event->key.keysym.sym == SDLK_LEFT)
  		{
  			if(this->focusedObject == -1)
  			{
  				this->focusedObject = this->objects.size() - 1;
  			}
  			else if(this->objects[this->focusedObject]->getType() != SGUI_TEXTFIELD && this->objects[this->focusedObject]->getType() != SGUI_DIALOG)
  			{
  				this->focusedObject--;
  				if(this->focusedObject < 0)
  				{
  					this->focusedObject = this->objects.size() -1;
  				}
  			}
  		}
  	}

		// if there is an object focused, tell him and hand over message
		if(this->focusedObject != -1)
		{
			if(this->focusedObject != lastFocusedObject)
			{
				this->objects[this->focusedObject]->focused = true;

				if(lastFocusedObject != -1)
				{
					this->objects[this->focusedObject]->focused = false;
				}
			}

#ifdef DEBUG
			cout << "Event handed over to object # " << this->focusedObject << endl;
#endif

			return this->objects[this->focusedObject]->message(event);
		}
	}

	// otherwise nothing happens
	return 42;
}
/** Sets the Color of the background by putting the values into the Background-Color Struct
Changes won't take effekt until after the next draw! */
int SGUI_Dialog::setBackgroundColor(Uint8 red, Uint8 green, Uint8 blue){
		this->backgroundColor.r = red;
	this->backgroundColor.g = green;
	this->backgroundColor.b = blue;
	return 42;
}
/** Will pop the Dialog with all it's Objects to the screen (when registered) */
int SGUI_Dialog::show(){
	// if already visible everything is fine
	if(this->visible)
		return 1;

	// make all objects visible
	for(unsigned int i = 0; i < this->objects.size(); i++)
		objects[i]->visible = true;

	this->visible = true;

	return 42;
}
/** Will hide the Dialog with all it's objects */
int SGUI_Dialog::hide(){
	if(!this->visible)
		return 1;

	// make all objects visible
	for(unsigned int i = 0; i < this->objects.size(); i++)
		objects[i]->visible = false;

	this->visible = false;

	return 42;

}
/** Allows to set the input focus on a certain object in the dialog
returns -1 if the given object does not exist in the dialog */
int SGUI_Dialog::setFocus(SGUI_Object * object){
	for(unsigned int i = 0; i < objects.size(); i++)
	{
		if(this->objects[i] == object)
		{
			// unselect old object
			if(this->focusedObject > 0 && this->focusedObject < this->objects.size())
				this->objects[this->focusedObject]->focused = false;

			// select new object
			this->focusedObject = i;
			this->objects[this->focusedObject]->focused = true;

			return 42;
		}
	}

	// nothing found
	return -1;

}
