/***************************************************************************
                          sgui_dialog.h  -  description
                             -------------------
    begin                : Son Feb 16 2003
    copyright            : (C) 2003 by Matthias Bach
    email                : marix@marix-world.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SGUI_DIALOG_H
#define SGUI_DIALOG_H

#include <vector>
#include <SDL/SDL.h>

#include "sgui_object.h"

// define Object-Type for this one
#define SGUI_DIALOG 0

/**Just an SGUI_Object that can contain other SGUI_Objects (kinda like a subwindow)
  *@author Matthias Bach
  */

class SGUI_Dialog : public SGUI_Object  {
public: 
	SGUI_Dialog();
	~SGUI_Dialog();
  /** Renders the dialog and all of it's objects and returns the Surface. Therefore it creates a SDL_Surface if needed. The return value is NULL on failure */
  SDL_Surface * draw();
  /** Sets the size to the given values. Returns 42 on succes and -1 on failure */
  int setSize(int w, int h);
  /** Removes an Object from the dialog */
  int removeObject(SGUI_Object * object);
  /** Adds an object to the dialog */
  int addObject(SGUI_Object * object);
  /** Handles the given event, mostly by passing it on to it's registered Objects */
  int message(SDL_Event * event);
  /** Sets the Color of the background by putting the values into the Background-Color Struct
Changes won't take effekt until after the next draw! */
  int setBackgroundColor(Uint8 red, Uint8 green, Uint8 blue);
  /** Will hide the Dialog with all it's objects */
  int hide();
  /** Will pop the Dialog with all it's Objects to the screen (when registered) */
  int show();
  /** Allows to set the input focus on a certain object in the dialog
returns -1 if the given object does not exist in the dialog */
  int setFocus(SGUI_Object * object);
protected: // Protected attributes
  /** The Objects of the Dialog */
  std::vector<SGUI_Object * >objects;
  /** Which object is "selected" right now */
  int focusedObject;
private: // Private attributes
  /**  */
  SDL_Color backgroundColor;
};

#endif
