/***************************************************************************
                          sgui_textfield.cpp  -  description
                             -------------------
    begin                : Son Feb 9 2003
    copyright            : (C) 2003 by Matthias Bach
    email                : marix@marix-world.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string>
#include <iostream>
#include <cctype>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
 
#include "sgui_textfield.h"

using namespace std;

SGUI_Textfield::SGUI_Textfield(){
	this->pxWidth = 100;
	this->pxHeight = 100;

	this->fontSize = 12;

	this->font = NULL;
}
SGUI_Textfield::~SGUI_Textfield()
{
	// make sure, that font is unloaded
	if(this->font != NULL)
		TTF_CloseFont(this->font);
}
/** sets blablabla as value */
int SGUI_Textfield::setText(string Text){
  this->text = Text;

  // text needs to be split into lines again
  if(this->splitTextIntoLines() < 0)
  {
  	cerr << "Text could not be splitted correctly after being set" << endl;
   	return -1;
  }

  // text needs to be rerendered
  if(this->renderText() < 0)
  {
  	cerr << "Text could not be rendered correctly after being set" << endl;
  	return -2;
  }
	
	return 42;
};

/** Returns the stuff currently wirtten to the field */
string SGUI_Textfield::getText(void){
	return this->text;
}

/** Add the Charecter to the Text */
int SGUI_Textfield::addToText(char c){
	this->text += c;

  // text needs to be split into lines again
  if(this->splitTextIntoLines() < 0)
  {
  	cerr << "Text could not be splitted correctly after adding charecter" << endl;
   	return -1;
  }

  // text needs to be rerendered
  if(this->renderText() < 0)
  {
  	cerr << "Text could not be rendered correctly after adding charecter" << endl;
  	return -2;
  }

  return 42;
}
/** As long as ther has no text been drawn, the object can be resized using this method */
int SGUI_Textfield::setSize(int iWidth, int iHeight){

	// if there is already something allocated, then error
	if(this->surface != NULL)
		return -1;

	this->pxHeight = iHeight;
	this->pxWidth = iWidth;

	// make sure, text will fit
	this->splitTextIntoLines();

	return 42;
}
/** Sets the color of the text. Range is from 0 to 255 each */
int SGUI_Textfield::setTextColor(Uint8 red, Uint8 green, Uint8 blue, Uint8 alpha){
	this->textColor.r = red;
	this->textColor.g = green;
	this->textColor.b = blue;

	// this does not really make sense, but it keeps the alpha somewhere
	this->textColor.unused = alpha;

  // text needs to be rerendered
  if(this->renderText() < 0)
  {
  	cerr << "Text could not be rendered correctly after changing color" << endl;
  	return -2;
  }
	

	return 42;	
}
/** Splits the Text of the textfield into lines that neatly fit into the window */
int SGUI_Textfield::splitTextIntoLines(){
	// make sure that the lines are empty
	lines.clear();
	
	// if there is no text, nothing can be done
	if(this->text == "")
	{
		return 1;
	}

	// multiple linebreaks at the end should be removed

	string oneLine = "";

	for(unsigned int i = 0; i < this->text.length(); i++)
	{
		// break the text, if next word won't fit. But one word will have to be in each line
		// this will be implemented later

		// break the text at linebreaks
		if(this->text[i] == '\n')
		{
			// add current line to list
			this->lines.push_back(oneLine);
			// make an empty line for the new one
			oneLine = "";
		}
		// if the char is printable add it to the current line
		else if(isprint(this->text[i]))
		{
				oneLine += this->text[i];
		}                            		
	}
	
	// add last lien, too
	this->lines.push_back(oneLine);

	return 42;
}


/** Sets the given font as Rendering-font */
int SGUI_Textfield::setFont(string fontName, int size){
	if(this->font != NULL)
		TTF_CloseFont(this->font);

	this->font = TTF_OpenFont(fontName.c_str(), size);

	if(this->font == NULL)
	{
		cerr << "Couldn't load Font " << fontName << endl;
		return -1;
	}
	
	this->fontName = fontName;
	this->fontSize = fontSize;
	this->lineHeight = TTF_FontLineSkip(this->font);

	TTF_SetFontStyle(this->font, TTF_STYLE_NORMAL);

	// the functions escape if no text was given, so this is no problem

  // text needs to be rerendered
  if(this->renderText() < 0)
  {
  	cerr << "Text could not be rendered correctly after setting font" << endl;
  	return -2;
  }


	return 42;
}


/** Renders the Text to the Surface */
int SGUI_Textfield::renderText(){
	// make sure there is a surface
	// the creating of the surface should not use fixed 32-bit depth. this will have to be fixed. also, some thought should be put
	// in whether SW or HW_SURFACE is the better flag
	// also there should be a more effective way of clearing the old surface

	if(this->surface == NULL)
	{                                                                                     
		this->surface = SDL_CreateRGBSurface(SDL_SRCCOLORKEY | SDL_RLEACCEL, this->pxWidth, this->pxHeight, 32, this->colorMask.Rmask, this->colorMask.Gmask, this->colorMask.Bmask, this->colorMask.Amask);
		if(this->surface == NULL)
		{
			cerr << "Error, surface could not be created" <<endl;
			return -1;
		}			
	}

	SDL_Color invisible;
/*	if(this->textColor.r < 127)
		invisible.r = 255;
	else
		invisible.r = 0;
	if(this->textColor.g < 127)
		invisible.g = 255;
	else
		invisible.g = 0;
	if(this->textColor.b < 127)
		invisible.b = 255;
	else
		invisible.b = 0;*/
		
	invisible.r = (this->textColor.r + 10) % 256;//(this->textColor.r + 128) % 256;
	invisible.g = (this->textColor.g + 10) % 256;//(this->textColor.g + 128) % 256;
  invisible.b = (this->textColor.b + 10) % 256;//(this->textColor.b + 128) % 256;

  
	// make colorkey exact opposite of text-color
	Uint32 colorkey = SDL_MapRGB(this->surface->format, invisible.r, invisible.g, invisible.b);

	// clear screen by blitting colorkey-color on it
	SDL_Rect dest;
	dest.x = 0;
	dest.y = 0;
	dest.w = this->surface->w;
	dest.h = this->surface->h;
	
	SDL_FillRect(this->surface, &dest, colorkey);
	// make the drawn color invisible
	SDL_SetColorKey(this->surface, SDL_SRCCOLORKEY | SDL_RLEACCEL, colorkey);


	//SDL_SetAlpha(this->surface, SDL_SRCALPHA | SDL_RLEACCEL, 255);      

	if(this->surface == NULL)
	{
		cerr << "Could not create Surface for Text" << endl;
		return -1;
	}

	// render the text

	// debug only
#ifdef DEBUG
	cout << "The lines are: " << endl;
	for(unsigned int line = 0; line < this->lines.size(); line++)
		cout << " " << this->lines[line] << endl;
	cout << endl;
#endif

	// make sure there is any text at all
	if(this->text == "")
		return 42;

	// memory needed for blitting
	SDL_Rect src;//, dest;
	SDL_Surface * tmp;

	src.x = 0;
	src.y = 0;

	dest.x = 0;
	dest.y = 0;

	for(unsigned int line = 0; line < this->lines.size(); line++)
	{
		if(this->lines[line] != "")
  	{
  		// render text
  		tmp = TTF_RenderText_Shaded(this->font, this->lines[line].c_str(), this->textColor, invisible);
  		//tmp = TTF_RenderText_Blended(this->font, this->lines[line].c_str(), this->textColor);

  #ifdef DEBUG
  		cout << "Rendered " << this->lines[line] << endl;
  #endif

  		if(tmp == NULL)
  		{
  			cerr << "Text was not rendered correctly!" << endl;
  			return -1;
  		}

  		// sizes for blitting fit
  		src.w = tmp->w;
  		src.h = tmp->h;
  		dest.w = src.w;
  		dest.h = src.h;

  		// blit to surface
  		SDL_BlitSurface(tmp, &src, this->surface, &dest);

  		// free tmp-surface to avoid memory leak
  		SDL_FreeSurface(tmp);
		}

		// make next line be blitted further down
		dest.y += this->lineHeight;
	}         


	// the text is now rendered
	return 42;
}
/** Sends an event to the Textfield.
The message will get handled */
int SGUI_Textfield::message(SDL_Event * event){
	// Textfields don't really care for messages
	// Only if they are writeable,
	// then they hunger for glyphs
	// or backspaces that will delete the last element
	if(this->writeable)
	{
		// check wether a key was stroken
		if(event->type == SDL_KEYDOWN)
		{
			SDL_keysym keysym;

			keysym = event->key.keysym;

			char ch;

			ch = this->translate(keysym);

			cout << ch << endl;
			cout << (int) ch << endl;

			if(isprint(ch))
				// add charecter to text
  			this->addToText(ch);

     	if(keysym.sym == SDLK_RETURN || keysym.sym == SDLK_KP_ENTER)
      	this->addToText('\n');

			// check for backspace charackter
			if(keysym.sym == SDLK_BACKSPACE)
			{
				// remove las character from text
				if(text.length() > 0)
  			{
  				this->text.erase(this->text.end() - 1);
  				// text needs to be split in lines again
  				this->splitTextIntoLines();
  				// text needs rerendering
  				this->renderText();
  			}
			}
				
		}
		// nothing besides keydown should be interesting
	}
	// and for sure nothing is interesting if the object is not writeable
	return 42;
}
		
		
/** Converts a pressed key into the corresponding character */
char SGUI_Textfield::translate(SDL_keysym keysym){
	char ch = 0;
	
 	if(keysym.sym == SDLK_a)
 		ch = 'A';
 	else if(keysym.sym == SDLK_b)
 		ch = 'B';
 	else if(keysym.sym == SDLK_c)
 		ch = 'C';
 	else if(keysym.sym == SDLK_d)
 		ch = 'D';
 	else if(keysym.sym == SDLK_e)
 		ch = 'E';
 	else if(keysym.sym == SDLK_f)
 		ch = 'F';
 	else if(keysym.sym == SDLK_g)
 		ch = 'G';
 	else if(keysym.sym == SDLK_h)
 		ch = 'H';
 	else if(keysym.sym == SDLK_i)
 		ch = 'I';
 	else if(keysym.sym == SDLK_j)
 		ch = 'J';
 	else if(keysym.sym == SDLK_k)
 		ch = 'K';
 	else if(keysym.sym == SDLK_l)
 		ch = 'L';
 	else if(keysym.sym == SDLK_m)
 		ch = 'M';
 	else if(keysym.sym == SDLK_n)
 		ch = 'N';
 	else if(keysym.sym == SDLK_o)
 		ch = 'O';
 	else if(keysym.sym == SDLK_p)
 		ch = 'P';
 	else if(keysym.sym == SDLK_q)
 		ch = 'Q';
 	else if(keysym.sym == SDLK_r)
 		ch = 'R';
 	else if(keysym.sym == SDLK_s)
 		ch = 'S';
 	else if(keysym.sym == SDLK_t)
 		ch = 'T';
 	else if(keysym.sym == SDLK_u)
 		ch = 'U';
 	else if(keysym.sym == SDLK_v)
 		ch = 'V';
 	else if(keysym.sym == SDLK_w)
 		ch = 'W';
 	else if(keysym.sym == SDLK_x)
 		ch = 'X';
 	else if(keysym.sym == SDLK_y)
 		ch = 'Y';
 	else if(keysym.sym == SDLK_z)
 		ch = 'Z';

  // check, whether this is no caps
  if(ch != 0 && !(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
  {
		ch += 32;
		return ch;
	}
   
	if(keysym.sym == SDLK_0)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '0';
  	else
   		ch = '=';
 	else if(keysym.sym == SDLK_1)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '1';
  	else
   		ch = '!';
 	else if(keysym.sym == SDLK_2)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '2';
  	else
   		ch = '"';
 	else if(keysym.sym == SDLK_3)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '3';
  	else
   		ch = '';
 	else if(keysym.sym == SDLK_4)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '4';
  	else
   		ch = '$';
 	else if(keysym.sym == SDLK_5)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '5';
  	else
   		ch = '%';
 	else if(keysym.sym == SDLK_6)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '6';
  	else
   		ch = '&';
 	else if(keysym.sym == SDLK_7)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '7';
  	else
   		ch = '/';
 	else if(keysym.sym == SDLK_8)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '8';
  	else
   		ch = '(';
 	else if(keysym.sym == SDLK_9)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '9';
  	else
   		ch = ')';
 	else if(keysym.sym == SDLK_COMMA)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = ',';
  	else
   		ch = ';';
 	else if(keysym.sym == SDLK_PERIOD)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '.';
  	else
   		ch = ':';
 	else if(keysym.sym == SDLK_KP_MINUS || keysym.sym == SDLK_MINUS)
		if(!(keysym.mod & KMOD_RSHIFT) && !(keysym.mod & KMOD_LSHIFT))
 			ch = '-';
  	else
   		ch = '_';
 	else if(keysym.sym == SDLK_SPACE)
 		ch = ' ';

  return ch;
}
