/***************************************************************************
                          sgui_window.cpp  -  description
                             -------------------
    begin                : Sam Feb 8 2003
    copyright            : (C) 2003 by Matthias Bach
    email                : marix@marix-world.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#include <iostream>


#include "sgui_window.h"

//#define DEBUG



using namespace std;

SGUI_Window::SGUI_Window(){
	// SDL needs to be initialized;
	if(SDL_Init(SDL_INIT_VIDEO) != 0)
		cout << "Error: Unable to initialize SDL: "<< SDL_GetError() << endl;

	// Init screen
	this->screen = NULL;

	// Initialize Font engine
	if(TTF_Init() != 0)
		cout << "Error: Unable to initialize TTF: " << SDL_GetError() << endl;

	// Load a font
	this->font = TTF_OpenFont("comic.ttf", 14);
	if(this->font == NULL)
		cout << "Error: Unable to load Font" << endl;

	if(this->font != NULL)
		TTF_SetFontStyle(this->font, TTF_STYLE_NORMAL);

	// Init Font-Color
	this->textColor.r = 0;
	this->textColor.b = 0;
	this->textColor.g = 255;
	this->textColor.unused = 0;

	this->visible = false;

	// init backgroundColor
	this->setBackgroundColor(0, 0, 0);

	// this is a gui, so we probably want key-repeat
	SDL_EnableKeyRepeat(250, 30);

	// in the beginning no object is focused
	this->focusedObject = -1;

	// init icon
	this->iconName = "";
	this->icon = NULL;

	// init windows name
	this->windowName = "SGUI Window";
}


SGUI_Window::~SGUI_Window(){
  // Make sure all object-memory is freed
  SGUI_Object * tmp;
  
  for(unsigned int i = 0; i < this->objects.size(); i++)
  {
		tmp = this->objects[i];
		if(tmp != NULL)
			delete tmp;
	}

	this->objects.clear();

	// Unload icon
	if(this->icon != NULL) {
		SDL_FreeSurface(icon);
	}
	
	// Unload Fonts
	if(this->font != NULL)
		TTF_CloseFont(this->font);

	// De-initialize the TTF engine
	TTF_Quit();
	
	// Unload SDL
	SDL_Quit();
}


/** This will blit the passed SDL_Surface to the Window (so it won't kill any other stuff) */
int SGUI_Window::blit(SDL_Surface * src, SDL_Rect * srcRect, SDL_Rect * dstRect){
	// offset due to menu's will have to be added

	// blit to screen
  if(SDL_BlitSurface(src, srcRect, this->screen, dstRect))
  {
		cout << "Error, couldn't blit to screen: " << SDL_GetError() << endl;
		return -1;
	}
	
	return 42;
}


/** Returns the height in px of the window that can acutally be drawn to */
int SGUI_Window::height(){
	// this is only for the beginning
	return this->realHeight;
}


/** Returns the width of the window that can acutally been drawn to
 */
int SGUI_Window::width(){
	// this is only for the beginning
	return this->realWidth;
}


/** Writes the given text to the screen using a standard Font */
int SGUI_Window::write(std::string text, int x, int y){

	if(this->font == NULL)
	{
		cout << "No font loaded!" << endl;
		return -1;
	}
	
	SDL_Surface * tmp;

	int width;
	int height;

	// get size and fill out src and dest info

	TTF_SizeText(this->font, text.c_str(), &width, &height);
	
	SDL_Rect srcRect;	
	srcRect.x = 0;
	srcRect.y = 0;
	srcRect.w = width;
	srcRect.h = height;

	SDL_Rect destRect;
	destRect.x = x;
	destRect.y = y;
	destRect.w = width;
	destRect.h = height;

	// render text 	
	tmp = TTF_RenderText_Blended(this->font, text.c_str(), this->textColor);
	if(tmp == NULL)
	{
		cout << "Error: Unable to render Text" << endl;
		return -1;
	}
                               
	// blit to screen
	this->blit(tmp, &srcRect, &destRect);	

	SDL_FreeSurface(tmp);
	return 42;
}
/** Sets the size of the Window
Return negative value on error */
int SGUI_Window::setSize(int x, int y){
	// if window is shown, this will have to be handled differently
	if(visible)
		return -1;
	
	this->realHeight = y;
	this->realWidth = x;

	return 42;
}
/** Puts the window on the Screen. */
int SGUI_Window::show(){
	// check whether window size has been specified
	if(this->realHeight < 1 || this->realWidth < 1)
		return -1;

  // check wheter icon was given and can be set
  if(this->screen == NULL && this->icon != NULL) {
    SDL_WM_SetIcon(this->icon, NULL);
	}

	// check whether a title was givne and can be set
	if(this->screen == NULL) {
    SDL_WM_SetCaption(this->windowName.c_str(), this->windowName.c_str());
	}
		
	this->screen = SDL_SetVideoMode(this->realWidth, this->realHeight, 32, SDL_DOUBLEBUF);
	if(this->screen == NULL)
		return -2;

	// draw background
	Uint32 color = SDL_MapRGB(this->screen->format, this->backgroundColor.r, this->backgroundColor.g, this->backgroundColor.b);

	SDL_Rect dest;
	dest.x = 0;
	dest.y = 0;
	dest.w = this->screen->w;
	dest.h = this->screen->h;

	SDL_FillRect(this->screen, &dest, color);
	this->draw();

	// window is now visible
	this->visible = true;

	return 42;
}
/** Draws the Window (including all buttons) */
int SGUI_Window::draw(){
  
  SDL_Rect src, dest;

  SDL_Surface * tmp;

	// objects need to be drawn
	for(unsigned int i = 0; i < this->objects.size(); i++)
	{
		if(this->objects[i]->visible)
		{
			src.x = 0;
			src.y = 0;
			src.w = this->objects[i]->width();
			src.h = this->objects[i]->height();

			dest.x = this->objects[i]->x;
			dest.y = this->objects[i]->y;
			dest.w = src.w;
			dest.h = src.h;

			

			tmp = this->objects[i]->draw();
			if(tmp != NULL)
				this->blit(tmp, &src, &dest);
		}
	};

	SDL_Flip(this->screen);

	// clear screen, so it is drawn to an empty screen afterwards
	Uint32 color = SDL_MapRGB(this->screen->format, this->backgroundColor.r, this->backgroundColor.g, this->backgroundColor.b);

	dest.x = 0;
	dest.y = 0;
	dest.w = this->screen->w;
	dest.h = this->screen->h;
	
	SDL_FillRect(this->screen, &dest, color);

	return 1;
}
/** Wait's for the next Event, remotes it to the hitten objects window and returns it's code, or the Event itself if it was not handled */
SDL_Event SGUI_Window::WaitEvent(){
	SDL_Event event;

	bool nothingHappened = true;

	// wait until something happens

	while(nothingHappened)
	{
		if(SDL_WaitEvent(&event))
		{
			// ignore Mousemotion
#ifdef DEBUG
			if(event.type != SDL_MOUSEMOTION && event.type != SDL_ACTIVEEVENT)
#else
			if(event.type != SDL_MOUSEMOTION)
#endif
			{
				nothingHappened = false;
				handleEvents(&event);
			}
		}
	}

	return event;
}
/** Check whether there was an event, and handles if nesecarry */
SDL_Event SGUI_Window::PeekEvent(){
	SDL_Event event;

	// check if something happend
	if(SDL_PollEvent(&event))
	{
		handleEvents(&event);
		return event;
	}

	// nothing happend
	event.type = SGUI_NO_EVENT;
	return event;
}
/** No descriptions */
int SGUI_Window::handleEvents(SDL_Event * event){

	int lastFocusedObject = this->focusedObject;

	// all of this does only make sense if there are objects
	if(this->objects.size() != 0)
	{
  	// if it was a mouse-click the focus may have changed
  	if(event->type == SDL_MOUSEBUTTONDOWN)
  	{
			// get click position
			int mouseX = event->button.x;
			int mouseY = event->button.y;

			
#ifdef DEBUG
			cout << "Mouse clicked (" << mouseX << "/" << mouseY << ")" << endl;
#endif
			
  		this->focusedObject = -1;
  		for(unsigned int i = 0; i < objects.size(); i++)
  		{
				// don't check object, if it is not visible
				if(objects[i]->visible)
				{
  				// check whether event was for that object
					if(mouseX > this->objects[i]->x && mouseX < this->objects[i]->x + this->objects[i]->width()
						&& mouseY > this->objects[i]->y && mouseY < this->objects[i]->y + this->objects[i]->height())
					{
						// this object is now focused
						this->focusedObject = i;

#ifdef DEBUG
						cout << "Object " << i << " hit" << endl;
#endif
					}
				}
  		}
  	}

  	// if it was a curser-key, and there is no object selected, or the selected object is not a text-field,
   	// on DOWN-ARRAY, RIGHT-ARRAY and TAB focus the next object
    // on UP-ARRAY and LEFT-ARRAY go back to the last object
    // don't go to next object, if current focused is a textfield or a dialog
  	if(event->type == SDL_KEYDOWN)
  	{
  		if(event->key.keysym.sym == SDLK_DOWN || event->key.keysym.sym == SDLK_RIGHT || event->key.keysym.sym == SDLK_TAB)
  		{
  			if(this->focusedObject == -1)
  			{
  				this->focusedObject = 0;
  			}
  			else if(this->objects[this->focusedObject]->getType() != SGUI_TEXTFIELD && this->objects[this->focusedObject]->getType() != SGUI_DIALOG)
  			{
  				this->focusedObject++;
  				this->focusedObject %= this->objects.size();
  			}
    	}
  		else if(event->key.keysym.sym == SDLK_UP || event->key.keysym.sym == SDLK_LEFT)
  		{
  			if(this->focusedObject == -1)
  			{
  				this->focusedObject = this->objects.size() - 1;
  			}
  			else if(this->objects[this->focusedObject]->getType() != SGUI_TEXTFIELD && this->objects[this->focusedObject]->getType() != SGUI_DIALOG)
  			{
  				this->focusedObject--;
  				if(this->focusedObject < 0)
  				{
  					this->focusedObject = this->objects.size() -1;
  				}
  			}
  		}
  	}

		// if there is an object focused, tell hime and hand over message
		if(this->focusedObject != -1)
		{
			if(this->focusedObject != lastFocusedObject)
			{
				this->objects[this->focusedObject]->focused = true;
				
				if(lastFocusedObject != -1)
				{
					this->objects[this->focusedObject]->focused = false;
				}
			}

#ifdef DEBUG
			cout << "Event handed over to object # " << this->focusedObject << endl;
#endif 
			
			return this->objects[this->focusedObject]->message(event);
		}
	}

	// otherwise nothing happens
	return 42;
}
/** An Object is added to the window (and the objects-vector). This will also be initialized to the windows display-format etc. */
int SGUI_Window::addObject(SGUI_Object * object){
	objects.push_back(object);
 
  if(screen != NULL)
  {
		object->setColorMask(screen->format->Rmask, screen->format->Gmask, screen->format->Bmask, screen->format->Amask);
	}

	object->visible = true;

	return 42;
}
/** Removes an Object from the SGUI_Window, but does not delete it! */
int SGUI_Window::removeObject(SGUI_Object * object){

	// find the object
	for(unsigned int i = 0; i < objects.size(); i++)
	{
		if(this->objects[i] == object)
		{
			// delete object
			
			object->focused = false;
			object->visible = false;
	
			this->objects.erase(this->objects.begin() + i);
			return 42;
		}
	}

	// object couln't be erased
	return -1;
}
/** Sets the Color of the background by putting the values into the Background-Color Struct */
int SGUI_Window::setBackgroundColor(Uint8 red, Uint8 green, Uint8 blue){
	this->backgroundColor.r = red;
	this->backgroundColor.g = green;
	this->backgroundColor.b = blue;
	return 42;
}
/** Allows to set the input focus on a certain object in the dialog
returns -1 if the given object does not exist in the dialog */
int SGUI_Window::setFocus(SGUI_Object * object){
	for(unsigned int i = 0; i < objects.size(); i++)
	{
		if(this->objects[i] == object)
		{
			// unselect old object
			if(this->focusedObject > 0 && this->focusedObject < this->objects.size())
				this->objects[this->focusedObject]->focused = false;

			// select new object
			this->focusedObject = i;
			this->objects[this->focusedObject]->focused = true;

			return 42;
		}
	}

	// nothing found
	return -1;
}

/** Setting of an icon */
int SGUI_Window::setIcon(std::string iconName) {
	// windows may not have been displayed yet
	if(this->screen != NULL) {
		cerr << "Icon cannot be set if the window is already displayed." << endl;
		return -2;
	}
	
	// TODO maybe it should be check whether the icon exists
	if(iconName == "") {
		cerr << "No icon specified!" << endl;
		return -1;
	}

	// set the icon-name (will be used on window-init)
	this->iconName = iconName;

	// make sure you don't leak memory
	if(this->icon != NULL) {
		SDL_FreeSurface(icon);
	}

	// load new icon
	this->icon = SDL_LoadBMP(this->iconName.c_str());

	// check if icon was loaded
	if(this->icon == NULL) {
		cerr << "Could not load icon " << this->iconName.c_str() << endl;
		cerr << "Reason: " << SDL_GetError() << endl;

		return -1;
	}

	return 0;
}

int SGUI_Window::setName(std::string windowName) {
	// windows may not have been displayed yet
	if(this->screen != NULL) {
		cerr << "Name cannot be set if the window is already displayed." << endl;
		return -2;
	}

	// set the icon-name
	this->windowName = windowName;

	return 0;
}

